@@ -193,27 +193,15 @@ XLALSimInspiralTaylorT1Setup(
     {
         case LAL_SIM_INSPIRAL_TIDAL_ORDER_ALL:
         case LAL_SIM_INSPIRAL_TIDAL_ORDER_6PN:
-            ak->akdEF.ETa6  = XLALSimInspiralPNEnergy_12PNTidalCoeff(
-                    ak->chi2, ak->chi1, lambda1)
-                    + XLALSimInspiralPNEnergy_12PNTidalCoeff(
-                    ak->chi1, ak->chi2, lambda2);
-            ak->akdEF.FTa12 = XLALSimInspiralPNFlux_12PNTidalCoeff(
-                    ak->chi1, lambda1)
-                    + XLALSimInspiralPNFlux_12PNTidalCoeff(
-                    ak->chi2, lambda2);
+	    ak->akdEF.ETa6  = lambda1 * XLALSimInspiralPNEnergy_12PNTidalCoeff(ak->chi1) + lambda2 * XLALSimInspiralPNEnergy_12PNTidalCoeff(ak->chi2);
+            ak->akdEF.FTa12 = lambda1 * XLALSimInspiralPNFlux_12PNTidalCoeff(ak->chi1) + lambda2 * XLALSimInspiralPNFlux_12PNTidalCoeff(ak->chi2);
         case LAL_SIM_INSPIRAL_TIDAL_ORDER_5PN:
-            ak->akdEF.ETa5  = XLALSimInspiralPNEnergy_10PNTidalCoeff(
-                    ak->chi2, ak->chi1, lambda1)
-                    + XLALSimInspiralPNEnergy_10PNTidalCoeff(
-                    ak->chi1, ak->chi2, lambda2);
-            ak->akdEF.FTa10 = XLALSimInspiralPNFlux_10PNTidalCoeff(
-                    ak->chi1, lambda1)
-                    + XLALSimInspiralPNFlux_10PNTidalCoeff(
-                    ak->chi2, lambda2);
+            ak->akdEF.ETa5  = lambda1 * XLALSimInspiralPNEnergy_10PNTidalCoeff(ak->chi1) + lambda2 * XLALSimInspiralPNEnergy_10PNTidalCoeff(ak->chi2);
+            ak->akdEF.FTa10 = lambda1 * XLALSimInspiralPNFlux_10PNTidalCoeff(ak->chi1) + lambda2 * XLALSimInspiralPNFlux_10PNTidalCoeff(ak->chi2);
         case LAL_SIM_INSPIRAL_TIDAL_ORDER_0PN:
             break;
         default:
-            XLALPrintError("XLAL Error - %s: Invalid tidal PN order %s\nSee LALSimInspiralTidalOrder enum in LALSimInspiralWaveformFlags.h for valid tidal orders.\n",
+            XLALPrintError("XLAL Error - %s: Invalid tidal PN order %d\nSee LALSimInspiralTidalOrder enum in LALSimInspiralWaveformFlags.h for valid tidal orders.\n",
                     __func__, tideO );
             XLAL_ERROR(XLAL_EINVAL);
             break;
@@ -302,7 +290,7 @@ int XLALSimInspiralTaylorT1PNEvolveOrbit(
 	double lengths, VRef = 0.;
 	int len, intreturn, idx, idxRef = 0;
 	XLALSimInspiralTaylorT1PNEvolveOrbitParams params;
-	ark4GSLIntegrator *integrator = NULL;
+	LALAdaptiveRungeKutta4Integrator *integrator = NULL;
 	expnFuncTaylorT1 expnfunc;
 	expnCoeffsTaylorT1 ak;
 
