@@ -20,11 +20,23 @@
 // Code for SWIG tests of the LALSimulation bindings.
 // Author: Karl Wette
 
-// Include LAL test code header.
 #include <lal/SWIGLALTest.h>
 
+#ifndef _SWIGLALSIMULATIONTEST_H
+#define _SWIGLALSIMULATIONTEST_H
+
+#ifdef  __cplusplus
+extern "C" {
+#endif
+
 // Test object parent tracking between modules.
 typedef struct tagswig_lalsimulation_test_parent_map_struct {
   swig_lal_test_struct s;
 } swig_lalsimulation_test_parent_map_struct;
-swig_lalsimulation_test_parent_map_struct swig_lalsimulation_test_parent_map;
+extern swig_lalsimulation_test_parent_map_struct swig_lalsimulation_test_parent_map;
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif // _SWIGLALSIMULATIONTEST_H
